package org.iphsoft.simon1.util;

import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.webkit.MimeTypeMap;
import android.widget.Toast;

import com.R;

import org.iphsoft.simon1.AndroidPortAdditions;
import org.iphsoft.simon1.AndroidPortAdditions.AppStore;

import java.io.File;

/**
 * Utils for dealing with system intents - launching system apps and so on
 * 
 * @author omergilad
 */
public class IntentUtils {

	public static final String AMAZON_DEVELOPER_URL = "amzn://apps/android?s=com.mojotouch";

	public static void showMailApp(Context context, String subject, String body, String email,
			boolean fromService)
	{
		Intent sendIntent = new Intent(Intent.ACTION_SEND);
		sendIntent.setType("plain/text");
		sendIntent.putExtra(Intent.EXTRA_SUBJECT, subject);
		sendIntent.putExtra(Intent.EXTRA_TEXT, body);
		sendIntent.putExtra(Intent.EXTRA_EMAIL, new String[]
		{ email });
		sendIntent = Intent.createChooser(sendIntent, "Select mail app");
		context.startActivity(sendIntent);
	}

	/**
	 * Show a URL in the native browser
	 * 
	 * @param url
	 * @param context
	 */
	public static void showBrowserApp(Context context, String url)
	{
		Intent i = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
		context.startActivity(i);
	}

	public static void showSmsApp(Context context, String body, String to, boolean fromService)
	{
		Intent sendIntent = new Intent(Intent.ACTION_VIEW);

		String uriString;
		if (to == null)
		{
			uriString = "sms:";
		} else
		{
			uriString = "smsto:" + to;
		}

		sendIntent.setData(Uri.parse(uriString));

		if (to != null)
		{
			sendIntent.putExtra("address", to);
		}
		sendIntent.putExtra("sms_body", body);

		context.startActivity(sendIntent);
	}

	public static void startAppStoreActivity(Context context, String packageName)
	{
		switch (AndroidPortAdditions.APP_STORE)
		{
		case GOOGLE_PLAY:
			context.startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(String.format(
					context.getString(R.string.google_play_market_url), packageName))));
			break;
		case AMAZON:
			context.startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(String.format(
					context.getString(R.string.amazon_market_url), packageName))));
			break;
		case SAMSUNG:
			Intent intent = new Intent();
			intent.setData(Uri.parse(String.format(context.getString(R.string.samsung_market_url),
					packageName)));
			context.startActivity(intent);
			break;
		default:
			MyLog.e("IntentUtils: startAppStoreActivity: unknown appstore");
		}
	}

	public static void startAppStoreDeveloperActivity(Context context)
	{
		switch (AndroidPortAdditions.APP_STORE)
		{
		case GOOGLE_PLAY:
			try
			{
				context.startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(context
						.getString(R.string.google_play_market_developer_url))));
			} catch (ActivityNotFoundException e)
			{
				context.startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(context
						.getString(R.string.google_play_market_developer_url_web))));
			}

			break;
		case AMAZON:
			try
			{
				context.startActivity(new Intent(Intent.ACTION_VIEW, Uri
						.parse(AMAZON_DEVELOPER_URL)));
			} catch (ActivityNotFoundException e)
			{
				Toast.makeText(context, R.string.no_amazon_appstore, Toast.LENGTH_LONG).show();
			}
			break;
		default:
			MyLog.e("IntentUtils: startAppStoreActivity: unknown appstore");
		}
	}

	public static void openFile(Context context, File file)
	{
		MyLog.d("IntentUtils: openFile: " + file.getAbsolutePath());

		String type = "";
		String extension = "";
		try
		{
			Intent intent = new Intent(android.content.Intent.ACTION_VIEW);
			//Uri data = Uri.parse("file://" + file.getAbsolutePath()); //deprecated
			Uri data = androidx.core.content.FileProvider.getUriForFile(context, context.getApplicationContext().getPackageName() + ".provider", file);
			int dot = file.getName().lastIndexOf(".");
			extension = file.getName().substring(dot + 1).toLowerCase();
			if (extension.equals("epub"))
			{
				type = "application/epub+zip";
			} else
			{
				type = MimeTypeMap.getSingleton().getMimeTypeFromExtension(extension);
			}

			if (type != null)
			{

				intent.setDataAndType(data, type);
				intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
				context.startActivity(intent);

				return;
			}

		} catch (Exception e)
		{
			e.printStackTrace();
		}

		String errorMsg;
		if (extension.equals("epub") || extension.equals("pdf"))
		{
			errorMsg = context.getResources().getString(R.string.error_opening_epub_file);
			if (AndroidPortAdditions.APP_STORE == AppStore.GOOGLE_PLAY)
			{
				startAppStoreActivity(context,
						context.getResources().getString(R.string.epub_reader_app_googleplay));

			} else if (AndroidPortAdditions.APP_STORE == AppStore.AMAZON)
			{
				showBrowserApp(context,
						context.getResources().getString(R.string.epub_reader_app_amazon));
			}
		} else if (extension.equals("ogg")|| extension.equals("aac"))
		{
			errorMsg = context.getResources().getString(R.string.error_opening_audio_file);
			if (AndroidPortAdditions.APP_STORE == AppStore.GOOGLE_PLAY)
			{
				startAppStoreActivity(context,
						context.getResources().getString(R.string.mp4_ogg_app_googleplay));

			} else if (AndroidPortAdditions.APP_STORE == AppStore.AMAZON)
			{
				showBrowserApp(context,
						context.getResources().getString(R.string.mp4_ogg_app_amazon));
			}
		} else
		{
			errorMsg = context.getResources().getString(R.string.error_opening_extra_file,
					file.getName());
		}
		Toast.makeText(context, errorMsg, Toast.LENGTH_LONG).show();
	}
}
